unit
  _Species;

interface

	uses
   	_Global;

	type
		PSpecies = ^TSpecies;
    TSpecies = object
    private
      N : Integer;
      rName : string [10];
      Icons : array [0 ..3] of PIcon;
      Commands : array [1 .. MaxCommands] of Integer;
    public
      constructor Init (TheName : string; TheId : Integer);
      procedure Read (FileName : string; Graph : Boolean);
      procedure AddCommand (TheCommand : string);
      function  Command (No : Integer) : Integer;
      function  Name : string;
      function  Icon (Index : Integer) : PIcon;
    end;

implementation

constructor TSpecies.Init;
begin
  N := 0;
  rName := TheName;
  FillChar(Icons, SizeOf(Icons), 0);
end;

function  TSpecies.Command;
begin
  Command := Commands[No];
end;

procedure TSpecies.AddCommand;
var
  Data : Integer;
  StrData : string;
  I : Integer;
begin
  Inc(N);
  for I := 1 to Length(TheCommand) do
    TheCommand[I] := UpCase(TheCommand[I]);

  if (TheCommand = 'HOP') then
    Commands[N] := ComHop
  else if (TheCommand = 'LEFT') then
    Commands[N] := ComLeft
  else if (TheCommand = 'RIGHT') then
    Commands[N] := ComRight
  else if (TheCommand = 'INFECT') then
    Commands[N] := ComInfect + 1 shl 8
  else
  begin
    I := Pos(' ', TheCommand);
    StrData := Copy(TheCommand, I + 1, 255);
    TheCommand[0] := Char(I - 1);
    Val(StrData, Data, I);
    Data := Data shl 8;
         if (TheCommand = 'INFECT'  ) then Commands[N] := Data + ComInfect
    else if (TheCommand = 'IFEMPTY' ) then Commands[N] := Data + IfEmpty
    else if (TheCommand = 'IFWALL'  ) then Commands[N] := Data + IfWall
    else if (TheCommand = 'IFSAME'  ) then Commands[N] := Data + IfSame
    else if (TheCommand = 'IFENEMY' ) then Commands[N] := Data + IfEnemy
    else if (TheCommand = 'IFRANDOM') then Commands[N] := Data + IfRandom
    else if (TheCommand = 'GO'      ) then Commands[N] := Data + Go;
  end;
end;

procedure ReadIcon (var Inf: file; var Icon : TIcon; X, Y : Integer);
var
  I : Integer;
  Buf : array [0 .. 255] of Byte;
begin
   Icon.X := IconSize - 1;
   Icon.Y := IconSize - 1;
   for I := Y - 1 downto 0 do
     if I < IconSize then
     begin
       BlockRead(Inf, Icon.Map[I], IconSize);
       if IconSize < X then
         BlockRead(Inf, Buf, X - IconSize);
     end
     else
       BlockRead(Inf, Buf, X);
end;

function  ReadBitmap (Name : string) : Pointer;
var
  Inf : file;
  I, J, X, Y : Integer;
  Buf : array [0 .. 53] of Byte;
  Bitmap : PIcon;
begin
  Assign(Inf, Name);
  Reset(Inf, 1);
  New(Bitmap);
  BlockRead(Inf, Buf, 54);
  X := ((Buf[18] + Buf[19] * 256) + 3) and not 3;
  Y := Buf[22] + Buf[23] * 256;
  {skip palette}
  Seek(Inf,  1024 + 54);
  ReadIcon(Inf, Bitmap^, X, Y);
  Close(Inf);
  ReadBitmap := Bitmap;
end;

procedure TSpecies.Read (FileName : string; Graph : Boolean);
var
  T : string;
  F : Text;
  I, J, K, L : Integer;
begin
  Assign(F, FileName);
  Reset(F);
  Readln(F, rName);
  Readln(F, T);
  Val(T, L, I);
  while not SeekEof(F) do
  begin
    Readln(F, T);
    AddCommand(T);
  end;
  Close(F);
  if Graph then
  begin
    Icons[L] := ReadBitmap(FileName + '.bmp');
    for K := 1 to 3 do
    begin
      New(Icons[(L + K) mod 4]);
      with Icons[(L + K) mod 4]^ do
      begin
        X := Icons[L]^.X;
        Y := Icons[L]^.Y;
        for I := 0 to IconSize - 1 do
          for J := 0 to IconSize - 1 do
            Map[I, J] := Icons[(L + K - 1) mod 4]^.Map[IconSize - 1 - J, I];
      end;
    end;
  end;
end;

function  TSpecies.Name;
begin
  Name := rName;
end;

function  TSpecies.Icon (Index : Integer) : PIcon;
begin
  Icon := Icons[Index];
end;

end.