unit
  _World;

interface

	uses
		_Global, _Creature, _Species;

  type
    PWorld = ^ TWorld;
   	TWorld = object
    private
      rN, rX, rY : Integer;
      World : array [0 .. MaxX + 1, 0 .. MaxY + 1] of PCreature;
      Screen : array [0 .. MaxX + 1, 0 .. MaxY + 1] of record
        Sp : PSpecies;
        Dir : TDirection;
      end;
      Queue : array [0 .. MaxX * MaxY - 1] of TPoint;
      Graph : Boolean;
    public
      constructor Init(TheX, TheY : Integer; Gr : Boolean);
      procedure Tick (I : Integer);
      function  AddCreature (TheX, TheY : Integer; Spc: PSpecies; Dir : TDirection) : Boolean;
      function  Element (TheX, TheY : Integer) : PCreature;
      function  N : Integer;
      function  X : Integer;
      function  Y : Integer;
      function  Graphic : Boolean;
      procedure InitDisplay;
      procedure Display (Pl : Integer);
    end;

implementation

uses
  CrtII, Graph;

constructor TWorld.Init;
begin
  rN := 0;
  rX := TheX;
  rY := TheY;
  Graph := Gr;
  FillChar(World, SizeOf(World), 0);
end;

function  TWorld.N;
begin
  N := rN;
end;

function  TWorld.X;
begin
  X := rX;
end;

function  TWorld.Y;
begin
  Y := rY;
end;

function  TWorld.Graphic;
begin
  Graphic := Graph;
end;

function  TWorld.Element;
begin
  Element := World[TheX, TheY];
end;

function  TWorld.AddCreature;
begin
  if (World[TheX, TheY] <> nil) then
  begin
    AddCreature := False;
    Exit;
  end;

  Inc(rN);
  Queue[rN].X := TheX;
  Queue[rN].Y := TheY;
  World[TheX, TheY] := New(PCreature, Init(Spc, Dir, 1));
  AddCreature := True;
end;

procedure TWorld.Tick;
var
  Ins : Integer;
  Xx, Yy, OldX, OldY : Integer;
  Cre, Front : PCreature;
  Dir : TDirection;
begin
  Xx := Queue[I].X;
  Yy := Queue[I].Y;
  Cre := World[Xx, Yy];
  Dir := Cre^.Direction^;
  OldX := Xx;
  OldY := Yy;
  Xx := Xx + dx[Dir];
  Yy := Yy + dy[Dir];

  if ((Xx < 1) or (Yy < 1) or (Xx > rX) or (Yy > rY)) then
  begin
    {There is wall in front}
    Ins := Cre^.DoInstruction(Wall);
  end
  else if (World[Xx, Yy] = nil) then
  begin
    {There is nothing in front}
	  Ins := Cre^.DoInstruction(Empty);
    {Creature hops}
    if (Ins = Hop) then
    begin
      Queue[I].X := Xx;
      Queue[I].Y := Yy;
      World[Xx, Yy] := World[OldX, OldY];
      World[OldX, OldY] := nil;
    end;
  end
  else
  begin
    Front := World[Xx, Yy];
    if (Front^.Species = Cre^.Species) then
    begin
      {There is a same type creature in front}
      Ins := Cre^.DoInstruction(Same);
    end
    else
    begin
      {Enemey is in front}
      Ins := Cre^.DoInstruction(Enemy);
      {Infect enemey in front}
      if (Ins and Infect = Infect) then
        World[Xx, Yy] := New(PCreature, Init(Cre^.Species, Front^.Direction^, Ins shr 8));
    end;
  end;
end;

var
  Pal : array [0 .. 767] of Byte;

procedure TWorld.InitDisplay;
var
  I, J : Integer;
  RGB : array [0 ..3] of Byte;
  F : file;
begin
  if Graph then
  begin
    FillChar(Screen, SizeOf(Screen), 255);
    I := InstallUserDriver('svga256', nil);
    if (rX * (IconSize + 1) < 640) and (rY * (IconSize + 1) < 480) then
      J := 2
    else
    if (rX * (IconSize + 1) < 800) and (rY * (IconSize + 1) < 600) then
      J := 3
    else
      J := 4;
    InitGraph(I, J, '');
    ClearDevice;
    Assign(F, 'palette.pal');
    Reset(F, 1);
    Seek(F, 54);
    for I := 0 to 255 do
    begin
      BlockRead(F, RGB, 4);
      Pal[3 * I + 0] := RGB[2] shr 2;
      Pal[3 * I + 1] := RGB[1] shr 2;
      Pal[3 * I + 2] := RGB[0] shr 2;
    end;
    Close(F);
    asm
      mov ax, 1012h
      mov bx, 0
      mov cx, 256
      mov dx, offset pal
      push seg pal
      pop es
      int 10h
    end;
  end
  else
  begin
    if (rX > 20) or (ry > 12) then
    begin
      Writeln('Too large table to fit in text mode.');
      Halt;
    end;
    asm
      mov ah, 1
      mov cx, 100h
      int 10h
    end;
    ClrScr;
    TextColor(1);
    DirectVideo := True;
    GotoXY(1, 1);
    Write('+--');
    for J := 1 to rX - 1 do
      Write('-+--');
    if rX < 20 then
      Write('-+');
    Writeln;
    for I := 1 to rY do
    begin
      Write('|  ');
      for J := 1 to rX - 1 do
        Write(' |  ');
      if rX < 20 then
        Write(' |');
      Writeln;
      Write('+--');
      for J := 1 to rX - 1 do
        Write('-+--');
      if rX < 20 then
        Write('-+');
      Writeln;
    end;
  end;
end;

procedure TWorld.Display;
var
  Id : Integer;
  X0, Y0, Xx, Yy, I, J, K : Integer;
  C : Char;
begin
  if Graph then
  begin
    X0 := (GetMaxX - rX * (IconSize + 1)) div 2;
    Y0 := (GetMaxY - rY * (IconSize + 1)) div 2;
    for I := 1 to rX do
      for J := 1 to rY do
        if (World[I, J]^.Species <> Screen[I, J].Sp) or
           (Longint(World[I, J]^.Direction^) <> Longint(Screen[I, J].Dir)) or
           (Pl >= 0) and (I = Queue[Pl].X) and (J = Queue[Pl].Y) then
        begin
          Xx := (I - 1) * (IconSize + 1) + X0;
          Yy := (J - 1) * (IconSize + 1) + Y0;
          Screen[I, J].Sp := World[I, J]^.Species;
          Screen[I, J].Dir := World[I, J]^.Direction^;
          if (World[I, J] <> nil) then
            World[I, J]^.Draw(Xx, Yy, (Pl >= 0) and (I = Queue[Pl].X) and (J = Queue[Pl].Y))
		      else
          begin
            SetFillStyle(1, 255);
            Bar(Xx, Yy, Xx + IconSize - 1, Yy + IconSize - 1);
          end;
          if (Pl >= 0) and (I = Queue[Pl].X) and (J = Queue[Pl].Y) then
            Screen[I, J].Sp := nil;
        end;
  end
  else
  begin
    TextColor(10);
    for I := 1 to rX do
    begin
      for J := 1 to rY do
      begin
        GotoXY(4 * I - 2, 2 * J);
        if (World[I, J] = nil) then
          Write('' : 3)
        else
          with World[I, J]^ do
          begin
            if (Pl >= 0) and (I = Queue[Pl].X) and (J = Queue[Pl].Y) then
              TextColor(12);
            Write(Copy(Species^.Name, 1, 2) : 2);
            case Direction^ of
              0 : C := Char(26);
              1 : C := Char(25);
              2 : C := Char(27);
              3 : C := Char(24);
            end;
            Write(C);
            if (Pl >= 0) and (I = Queue[Pl].X) and (J = Queue[Pl].Y) then
              TextColor(10);
          end;
      end;
    end;
    GotoXY(80,25);
  end;
end;

end.
